@@ -4,6 +4,7 @@ import json
 import logging
 import time
 import uuid
+import re
 from collections import OrderedDict
 
 from wzdx.tools import date_tools, polygon_tools, wzdx_translator
@@ -18,10 +19,17 @@ STRING_DIRECTION_MAP = {'north': 'northbound', 'south': 'southbound',
 REVERSED_DIRECTION_MAP = {'northbound': 'southbound', 'southbound': 'northbound',
                           'eastbound': 'westbound', 'westbound': 'eastbound'}
 
+WORK_ZONE_INCIDENT_TYPES = {
+    "Maintenance Operations":   {"Traffic": True},
+    "Other":                    {"Environmental": True},
+    "Emergency Roadwork":       {"Traffic": True},
+}
+INCIDENT_ID_REGEX = "^OpenTMS-Incident"
+
 
 def main():
-    navjoy_file, output_dir = parse_rtdh_arguments()
-    input_file_contents = open(navjoy_file, 'r').read()
+    source_file, output_dir = parse_rtdh_arguments()
+    input_file_contents = open(source_file, 'r').read()
     generated_messages = generate_standard_messages_from_string(
         input_file_contents)
 
@@ -64,14 +72,35 @@ def generate_standard_messages_from_string(input_file_contents):
     return standard_messages
 
 
+# TODO: Integrate Category
+def is_incident(msg):
+    id = msg.get('properties', {}).get('id', '')
+    type = msg.get('properties', {}).get('type', '')
+    # category = msg.get('properties', {}).get('Category')
+    is_incident = re.match(INCIDENT_ID_REGEX, id) != None
+    # is_wz = WORK_ZONE_INCIDENT_TYPES.get(type, {}).get(category)
+    is_wz = WORK_ZONE_INCIDENT_TYPES.get(type, {}) != {}
+    return is_incident, is_wz
+
+
 def generate_raw_messages(message_string):
     msg = json.loads(message_string)
     messages = []
 
     separated_messages = expand_event_directions(msg)
     for indiv_msg in separated_messages:
-        if validate_closure(indiv_msg):
+        is_incident_msg, is_wz = is_incident(indiv_msg)
+        if is_incident_msg and is_wz:
             messages.append(indiv_msg)
+        elif not is_incident_msg:
+            messages.append(indiv_msg)
+        else:
+            id = indiv_msg.get('properties', {}).get('id')
+            dir = indiv_msg.get('properties', {}).get('direction')
+            logging.warning(
+                f"Ignoring non-work zone incident: {id}_{dir}")
+            # Don't process non-wz incidents
+            pass
 
     return messages
 
@@ -101,9 +130,9 @@ def expand_event_directions(message):
         return [message]
 
 
-def generate_rtdh_standard_message_from_raw_single(obj):
+def generate_rtdh_standard_message_from_raw_single(obj, is_incident=False):
     pd = PathDict(obj)
-    standard_message = create_rtdh_standard_msg(pd)
+    standard_message = create_rtdh_standard_msg(pd, is_incident)
     return standard_message
 
 
@@ -154,6 +183,7 @@ def hex_to_binary(hex_string):
 # TODO: Consider support road closures
 DEFAULT_EVENT_TYPE = ('work-zone', [])
 EVENT_TYPE_MAPPING = {
+    # Work Zones
     "Bridge Construction":              ('work-zone', [{'type_name': 'below-road-work',            'is_architectural_change': True}]),
     "Road Construction":                ('work-zone', [{'type_name': 'roadway-creation',           'is_architectural_change': True}]),
     "Bridge Maintenance Operations":    ('work-zone', [{'type_name': 'below-road-work',            'is_architectural_change': False}]),
@@ -183,6 +213,7 @@ EVENT_TYPE_MAPPING = {
     "Wall Maintenance":                 ('work-zone', [{'type_name': 'barrier-work',               'is_architectural_change': False}]),
     "Other":                            ('work-zone', []),
 
+    # Road Closures
     "BAN Message":                      ('restriction', []),
     "Safety Campaign":                  ('restriction', []),
     "Smoke/Control Burn":               ('restriction', []),
@@ -194,6 +225,10 @@ EVENT_TYPE_MAPPING = {
     "Local Event":                      ('restriction', []),
     "Military Movement":                ('restriction', []),
     "OS/OW Limit":                      ('restriction', []),
+
+    # Incidents (work zones)
+    "Emergency Roadwork":               ('work-zone', []),
+    "Maintenance Operations":           ('work-zone', []),
 }
 
 
@@ -203,8 +238,7 @@ LANE_TYPE_MAPPING = {
     "center lane": 'general',
     "middle two lanes": 'general',
     "general": 'general',
-    # this is a weird one
-    "middle lanes": 'general',
+    "middle lanes": 'general',     # this is a weird one
     "right lane": 'general',
     "right shoulder": 'shoulder',
     "through lanes": 'general',
@@ -287,13 +321,27 @@ def all_lanes_open(lanes):
     return True
 
 
-def create_rtdh_standard_msg(pd):
-    if pd.get('properties/travelerInformationMessage') == INVALID_EVENT_DESCRIPTION:
-        logging.warning(
-            f"Invalid message with id: '{pd.get('properties/id')}'' because description matches invalid event description: '{INVALID_EVENT_DESCRIPTION}'")
-        return {}
+# On {roadName}, between mile markers {startMarker} and {endMarker}. {typeOfWork}. Running between {startTime} and {endTime}
+def create_description(name, roadName, startMarker, endMarker, typeOfWork, startTime, endTime):
+    return f"Event {name}, on {roadName}, between mile markers {startMarker} and {endMarker}. {typeOfWork}. Running between {startTime} and {endTime}"
+
+
+def create_rtdh_standard_msg(pd, isIncident):
+    description = pd.get('properties/travelerInformationMessage')
+    if description == INVALID_EVENT_DESCRIPTION:
+        description = create_description(
+            pd.get('properties/name'),
+            pd.get('properties/routeName'),
+            pd.get('properties/startMarker'),
+            pd.get('properties/endMarker'),
+            pd.get('properties/type'),
+            pd.get('properties/startTime'),
+            pd.get('properties/clearTime'),
+        )
 
     coordinates = get_linestring(pd.get('geometry', default={'type': None}))
+    if not coordinates:
+        return {}
 
     direction = pd.get("properties/direction", default='unknown')
 
